// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function n = matmul_searchforsize ( timemin , nfact )
    // Returns the size of the matrix for which the A*B product is larger than timemin
    //
    // Example
    // stacksize("max");
    // n = matmul_searchforsize ( 1 , 1.2 )

    //
    // Let the size of the matrix grow dynamically

    backrand = rand("info")
    rand( "normal" );

    //
    // Make a loop over n
    n = 1;
    k = 1;
    while ( %t )
        A = rand(n,n);
        B = rand(n,n);
        tic();
        ierr = execstr("C = A * B","errcatch");
        t = toc();
        if ( ierr <> 0 ) then
            laerr = lasterror();
            msg = sprintf(gettext("%s: Error while running benchmark: %s."),..
              "matmul_searchforsize",laerr);
            warning(msg)
            break
        end
        if ( t > timemin ) then
            break
        end
        n = ceil(nfact * n);
    end
    rand(backrand)
endfunction

function [n,t,mflops] = matmul_histogram ( totaltime , kmax )
    // Performs the matrix-matrix product kmax times, up to totaltime seconds.
    // Returns the table of times t and the table of mflops.
    // This function may be used to see if the performances can be reproduced.

    // The minimum time for a run
    timemin = totaltime/kmax;
    mprintf("Minimum t = %f\n",timemin)
    // The update factor for the matrix size
    nfact = 1.2;
    // Get a reasonable matrix size
    mprintf("Searching for a matrix size...\n")
    n = matmul_searchforsize ( timemin , nfact );
    mprintf("Matrix size n=%d\n",n)
    for k = 1 : kmax
        A = rand(n,n);
        B = rand(n,n);
        tic();
        C = A * B;
        t(k) = toc();
        mflops(k) = floor(2*n^3/t(k)/1.e6);
        mprintf("Run #%d/%d: t = %f, mflops=%d\n",k,kmax,t(k),mflops(k))
    end
endfunction


stacksize("max");
// The total amount of time (seconds) require by the script
totaltime = 60;
// The number of times that the experiment is repeated
kmax = 200;
[n,t,mflops] = matmul_histogram ( totaltime , kmax );
scf();
subplot(1,2,1);
histplot(10,t)
xtitle("Matrix-Matrix product - n="+string(n),"Time (s)","Number of runs")
subplot(1,2,2);
histplot(10,mflops)
xtitle("Matrix-Matrix product - n="+string(n),"Megaflops","Number of runs")

